@@ -89,6 +89,14 @@ QT_BEGIN_NAMESPACE
     graphics hardware. If this is the case, the \l renderMode property can be used to
     switch how the View3D is rendered into the window.
 
+    A View3D with the default Offscreen \l renderMode is implicitly a
+    \l{QSGTextureProvider}{texture provider} as well. This means that \l
+    ShaderEffect or \l{QtQuick3D::Texture::sourceItem}{Texture.sourceItem} can reference
+    the View3D directly as long as all items live within the same
+    \l{QQuickWindow}{window}. Like with any other \l Item, it is also possible
+    to switch the View3D, or one of its ancestors, into a texture-based
+    \l{QtQuick::Item::layer.enabled}{item layer}.
+
     \sa {Qt Quick 3D - View3D example}
 */
 
@@ -118,10 +126,7 @@ QQuick3DViewport::~QQuick3DViewport()
     delete m_sceneRoot;
     m_sceneRoot = nullptr;
 
-    // We need to defere the deletion of the scene manager or items might
-    // try to deref using the now destroyed scene manaager.
-    connect(this, &QObject::destroyed, [sceneManager](QObject *) { delete sceneManager; });
-    sceneManager = nullptr;
+    delete sceneManager;
 
     // m_directRenderer must be destroyed on the render thread at the proper time, not here.
     // That's handled in releaseResources() + upon sceneGraphInvalidated
@@ -295,7 +300,41 @@ QQuick3DRenderStats *QQuick3DViewport::renderStats() const
 
 QQuick3DSceneRenderer *QQuick3DViewport::createRenderer() const
 {
-    return new QQuick3DSceneRenderer(this->window());
+    QQuick3DSceneRenderer *renderer = nullptr;
+
+    if (QQuickWindow *qw = window()) {
+        QSGRendererInterface *rif = qw->rendererInterface();
+        const bool isRhi = QSGRendererInterface::isApiRhiBased(rif->graphicsApi());
+        if (isRhi) {
+            QRhi *rhi = static_cast<QRhi *>(rif->getResource(qw, QSGRendererInterface::RhiResource));
+            if (!rhi)
+                qWarning("No QRhi from QQuickWindow, this cannot happen");
+
+            // The RenderContextInterface, and the objects owned by it (such
+            // as, the BufferManager) are always per-QQuickWindow, and so per
+            // scenegraph render thread. Hence the association with window.
+            // Multiple View3Ds in the same window can use the same rendering
+            // infrastructure (so e.g. the same QSSGBufferManager), but two
+            // View3D objects in different windows must not, except for certain
+            // components that do not work with and own native graphics
+            // resources (most notably, QSSGShaderLibraryManager - but this
+            // distinction is handled internally by QSSGRenderContextInterface).
+            if (QSSGRenderContextInterface *rci = QSSGRenderContextInterface::renderContextForWindow(*qw)) {
+                renderer = new QQuick3DSceneRenderer(rci);
+            } else {
+                QSSGRef<QSSGRhiContext> rhiContext(new QSSGRhiContext);
+                // and this is the magic point where many things internally get
+                // switched over to be QRhi-based.
+                rhiContext->initialize(rhi);
+                rci = new QSSGRenderContextInterface(qw, rhiContext, QString::fromLatin1("./"));
+                renderer = new QQuick3DSceneRenderer(rci);
+            }
+
+            QObject::connect(qw, &QQuickWindow::afterFrameEnd, this, &QQuick3DViewport::cleanupResources);
+        }
+    }
+
+    return renderer;
 }
 
 bool QQuick3DViewport::isTextureProvider() const
@@ -446,7 +485,10 @@ QSGNode *QQuick3DViewport::updatePaintNode(QSGNode *node, QQuickItem::UpdatePain
 
         const QSize targetSize = window()->effectiveDevicePixelRatio() * QSize(width(), height());
 
-        if (isVisible() && isComponentComplete()) {
+        // checkIsVisible, not isVisible, because, for example, a
+        // { visible: false; layer.enabled: true } item still needs
+        // to function normally.
+        if (checkIsVisible() && isComponentComplete()) {
             n->renderer->synchronize(this, targetSize, window()->effectiveDevicePixelRatio(), false);
             updateDynamicTextures();
             n->markDirty(QSGNode::DirtyMaterial);
@@ -676,6 +718,25 @@ void QQuick3DViewport::invalidateSceneGraph()
     m_node = nullptr;
 }
 
+void QQuick3DViewport::cleanupResources()
+{
+    // Pass the scene managers list of resouces marked for
+    // removal to the render context for deleation
+    // The render contect will take ownership of the nodes
+    // and clear the list
+    if (auto renderer = getRenderer()) {
+        const auto &rci = renderer->m_sgContext;
+        if (m_sceneRoot) {
+            const auto sceneManager = QQuick3DObjectPrivate::get(m_sceneRoot)->sceneManager;
+            rci->cleanupResources(sceneManager->resourceCleanupQueue);
+        }
+        if (m_importScene) {
+            const auto importSceneManager = QQuick3DObjectPrivate::get(m_importScene)->sceneManager;
+            rci->cleanupResources(importSceneManager->resourceCleanupQueue);
+        }
+    }
+}
+
 QQuick3DSceneRenderer *QQuick3DViewport::getRenderer() const
 {
     QQuick3DSceneRenderer *renderer = nullptr;
